/* ============================================================
 * File  : streamtuned.h
 * Author: Eric Giesselbach <ericgies@kabelfoon.nl>
 * Date  : 2004-02-12
 * Description : streamtuned gui
 *
 * Copyright 2003 by Eric Giesselbach

 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

// development snapshot, many todo's

#ifndef STREAMTUNED_H
#define STREAMTUNED_H

#include <qsqldatabase.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qrect.h>
#include <qtimer.h>
#include <qintdict.h>
#include <qpointarray.h>
#include <qbitmap.h>
#include <qsettings.h>


#include "../libs/streambrowser.h"
#include "../libs/streamconfig.h"
#include "../libs/storageconfig.h"
#include "../libs/streamstorage.h"
#include "guielements.h"


class ConfigWriter : public QSettings
{
  public:
    ConfigWriter();
    QColor readColor(QString key, uint defColor);
    uint   readUInt (QString key, uint defVal  );
};

//--------------------------------------------------------------------------------------------

class BrowserDisplay : public QWidget
{
  Q_OBJECT
  
  public:
    BrowserDisplay(QWidget *parent, GuiParameters* params, QRect rect, QIntDict<FolderLabel> &labels);
    ~BrowserDisplay();
      
    virtual void setItemIcon(int index, QChar& idch);
    virtual void setItemLabel(int index, QString& txt, bool selected);
    virtual void setFolderLabel(int index, QString& txt, bool selected);
    virtual void setTitleLabel(QString& txt);
    virtual void setTitleColor(QColor color, bool bold);
    virtual uint getItemCount();
    virtual uint getFolderCount();
   
    // make private
    QIntDict<ItemLabel>   itemLabels;
    QIntDict<ItemIcon>    itemIcons;
    QIntDict<FolderLabel> folderLabels;
  
  signals:
    void getIconPixmap( QPixmap& pix, QChar prefix );  // get pixmap based on prefix
        
  private:
    GuiParameters *guiParameters;
    ItemLabel *titleLabel;
};


class StreamDisplay : public QWidget
{
  Q_OBJECT
  
  public:
    StreamDisplay(QWidget *parent, GuiParameters* params, QRect rect, QIntDict<FolderLabel> &labels);
      
    virtual void setFolderLabel(int index, QString& txt, bool selected);
    virtual void setTitleColor(QColor color, bool bold);
    virtual void setTitleLabel(QString& txt);
    virtual uint getFolderCount();
   
    // make private
    QIntDict<FolderLabel> folderLabels;
    ItemLabel *streamName, *streamDescr;
    ItemLabel *streamAudioFormat, *streamAudioQuality;
    ItemLabel *streamVideoFormat;

    HarvesterMessageBox *harvesterMessageBox;
    PlayerMessageBox    *playerMessageBox;

    ItemLabel *streamTime, *streamTimeLabel;
    ItemLabel *streamStatus, *streamStatusLabel;

    ItemLabel *streamStabilityLabel, *streamCacheLabel;
    Gauge *streamStabilityGauge, *streamCacheGauge;

    FFTBox *fft;
          
  private:
    GuiParameters *guiParameters;
    ItemLabel *titleLabel;
};


//--------------------------------------------------------------------------------------------

class GenericHandler : public QObject
{  
  Q_OBJECT
  
  public:
    GenericHandler() {};
  
    virtual bool handlePressedKey( int key ) = 0;
    virtual void activate() = 0;
    virtual void deactivate() = 0;
    virtual void setTitleColor(QColor color) = 0;
    
  signals:
    void switchHandler( int key );  
};


class BrowserHandler : public GenericHandler
{
   Q_OBJECT
   
   public:
      BrowserHandler(BrowserDisplay *browserDisplay, StreamBrowser *streamBrowser);
      
      bool handlePressedKey( int key );
      void activate();
      void deactivate();      
      void setTitleColor(QColor color);
   
      // make private!
      void getFolderList();
      void getItemList();
   private:

      BrowserDisplay *browserDisplay;
      StreamBrowser *streamBrowser;
};


class StreamHandler : public GenericHandler
{
   Q_OBJECT
   
   public:
      StreamHandler(StreamDisplay *streamDisplay, StreamBrowser *streamBrowser);
      
      bool handlePressedKey( int key );
      void activate();
      void deactivate();      
      void setTitleColor(QColor color);
      void updateStreamStatusValues();
      void updateStreamPlayValues();
      void harvesterBusySignal(bool busy, QString& message);
   
      // make private!
   private:
      StreamDisplay *streamDisplay;
      StreamBrowser *streamBrowser;
};

/*
class ViewerHandler : public GenericHandler
{
   Q_OBJECT
   
   public:
      ViewerHandler(ViewerDisplay *viewerDisplay, StreamBrowser *streamBrowser);
      
      bool handlePressedKey( int key );
      void activate();
      void deactivate();      
      void setTitleColor(QColor color);
      
      // make private!
   private:
      void navigateItems();

      BrowserDisplay *browserDisplay;
      StreamBrowser *streamBrowser;
      QStringList displist, metalist, viewerlist;
      bool dataLoaded;
      QString title;
      uint displaycursor, listtop;

};
*/
//--------------------------------------------------------------------------------------------

class StreamTuned : public QWidget
{
  Q_OBJECT

  public:
    StreamTuned( QWidget *parent=0, const char *name=0 );
    ~StreamTuned();

    void updateBackground();
    void keyPressEvent( QKeyEvent *k );
    void mousePressEvent( QMouseEvent * e );
    void mouseReleaseEvent(  QMouseEvent * e );
    void mouseMoveEvent(  QMouseEvent * e );

  protected:
    void focusInEvent ( QFocusEvent * );
    void focusOutEvent ( QFocusEvent * );

  private slots:
    void slotItemTreeSwitchedTo(int);
    void slotHarvesterBusy(bool busy, QString message);
    void slotVideoActive(bool active);
    void slotUserMessage();
    void slotValuesUpdated(int updatedType);
    void slotStorageEvent(int ident, int eventType, bool error);
    void slotRecorderActive(bool active);
    void slotGetIconPixmap(  QPixmap& pix, QChar prefix );
    void slotSwitchHandler(int key);
    
  private:

    SampleObject *sampler;

    Led *streamLed, *harvesterLed;
    RecLed * recLed;
    Logo *logo;

    MessageLabel *userMessageBox;

    QRect itemButtonArea, folderButtonArea, globalButtonArea,
          controlButtonArea;
    QRect videoRect, videoMaxRect;
    QRect viewerRect, viewerMaxRect;

    QSqlDatabase *db;
    StreamBrowser *streamBrowser;

    void reportEvent(QString msg);
    void setDisplayStatus(const QString& status);
    bool pollResetDisplay(bool initialize);
    int  pollResetDisplayCount;

    bool copyFile(const QString& source, const QString& dest);
    bool createStorageRepos(const QString& dest, const QString& path);

    void selectFont();

    void loadActualBackground();
        
    void switchHandler(GenericHandler *handler);

    // state
    QString displayStatus;       // audio, video, browse
    QString currentStreamStatus; // idle, buffering, playing, paused, stopping, stopped, nostream
    bool harvesterBusy;          // harvester is fetching data
    bool videoActive;

    StreamBrowser::ItemTreeLoaded itemTreeLoaded; 
    
    StreamStorage *streamStorage;
    ReposStorage *storageBin;
    StreamTunedConfig *streamTunedConfig;

    QPoint mousePosition;

    QPixmap pict;

    QPixmap streamPict, urlPict, unknownPict, scheduledPict, infoPict, htmlPict,
            recordingPict, recordedPict, downloadPict, generalPict, headerPict, markedPict;
            
    ConfigWriter settings;
    GuiParameters *guiParameters;
    QIntDict<FolderLabel> folderLabels;
    
    BrowserDisplay *browserDisplay;
    StreamDisplay  *streamDisplay;

    bool switchLoopDetect;    
    BrowserHandler   *browserHandler;
    StreamHandler    *streamHandler;
    GenericHandler   *viewerHandler;
    
    GenericHandler   *currentHandler;
    GenericHandler   *previousHandler;
};


#endif /* STREAMTUNED_H */
